
    (*********************************************************************
     *  MODFilter.pas                                                    *
     *                                                                   *
     *  author    : Milenko Mitrovic                                     *
     *  email     : dcoder@dsp-worx.de                                   *
     *  web       : http://dsp-worx.de                                   *
     *  date      : 31-10-2003                                           *
     *  version   : 0.60                                                 *
     *                                                                   *
     *  The contents of this file are used with permission, subject to   *
     *  the Mozilla Public License Version 1.1 (the "License"); you may  *
     *  not use this file except in compliance with the License. You may *
     *  obtain a copy of the License at                                  *
     *  http://www.mozilla.org/MPL/MPL-1.1.html                          *
     *                                                                   *
     *  Software distributed under the License is distributed on an      *
     *  "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   *
     *  implied. See the License for the specific language governing     *
     *  rights and limitations under the License.                        *
     *                                                                   *
     *  (C) 2003 Milenko Mitrovic <dcoder@dsp-worx.de>                   *
     *                                                                   *
     *********************************************************************)

unit MODFilter;

interface

uses
  BaseClass, DirectShow9, ActiveX, Windows, SysUtils, MMSystem, Registry,
  MODDecoder, DSUtil, dmoconst;

const
  CLSID_MODSource : TGUID = '{C5C8E44B-19D1-4AAD-91C5-4903867F71DA}';

  MyPinType: TRegPinTypes = (
    clsMajorType: @MEDIATYPE_Audio; clsMinorType: @MEDIASUBTYPE_PCM
  );

  MyPins : TRegFilterPins = (
    strName: 'Output'; bRendered: FALSE; bOutput: TRUE; bZero: FALSE; bMany: FALSE; oFilter: nil; strConnectsToPin: nil; nMediaTypes: 1; lpMediaType: @MyPinType
  );

type
  TMODSourceStream = class(TBCSourceStream, IMediaSeeking)
  private
    FDecoder: TMODDecoder;
    FBufferSize: Integer;
    FRateSeeking: Double;
    FSeekingCaps: DWORD;
    FDuration: Int64;
    FStart: Int64;
    FStop: Int64;
    FDiscontinuity: Boolean;
    FSampleTime: Int64;
    FMediaTime: Int64;
    FLock: TBCCritSec;
    function ChangeStart: HRESULT;
    function ChangeStop: HRESULT;
    function ChangeRate: HRESULT;
    procedure UpdateFromSeek;
    function GetTimeFromBytes(Bytes: Int64): TReferenceTime;
  public
    constructor Create(const ObjectName: string; out hr: HRESULT; Filter: TBCSource; const Name: WideString; Filename : PWCHAR);
    destructor Destroy; override;
    function GetMediaType(MediaType: PAMMediaType): HRESULT; override;
    function FillBuffer(Samp: IMediaSample): HRESULT; override;
    function DecideBufferSize(Allocator: IMemAllocator; Properties: PAllocatorProperties): HRESULT; override;
    function NonDelegatingQueryInterface(const IID: TGUID; out Obj): HResult; override; stdcall;
    function OnThreadStartPlay: HRESULT; override;
    // IMediaSeeking
    function GetCapabilities(out pCapabilities: DWORD): HResult; stdcall;
    function CheckCapabilities(var pCapabilities: DWORD): HResult; stdcall;
    function IsFormatSupported(const pFormat: TGUID): HResult; stdcall;
    function QueryPreferredFormat(out pFormat: TGUID): HResult; stdcall;
    function GetTimeFormat(out pFormat: TGUID): HResult; stdcall;
    function IsUsingTimeFormat(const pFormat: TGUID): HResult; stdcall;
    function SetTimeFormat(const pFormat: TGUID): HResult; stdcall;
    function GetDuration(out pDuration: int64): HResult; stdcall;
    function GetStopPosition(out pStop: int64): HResult; stdcall;
    function GetCurrentPosition(out pCurrent: int64): HResult; stdcall;
    function ConvertTimeFormat(out pTarget: int64; pTargetFormat: PGUID; Source: int64; pSourceFormat: PGUID): HResult; stdcall;
    function SetPositions(var pCurrent: int64; dwCurrentFlags: DWORD; var pStop: int64; dwStopFlags: DWORD): HResult; stdcall;
    function GetPositions(out pCurrent, pStop: int64): HResult; stdcall;
    function GetAvailable(out pEarliest, pLatest: int64): HResult; stdcall;
    function SetRate(dRate: double): HResult; stdcall;
    function GetRate(out pdRate: double): HResult; stdcall;
    function GetPreroll(out pllPreroll: int64): HResult; stdcall;
  end;

  TMODSource = class(TBCSource, IFileSourceFilter, IAMMediaContent, IPersist)
  private
    FPin: TMODSourceStream;
    FFileName: PWChar;
  public
    constructor Create(const Name: string; unk: IUnknown; const clsid: TGUID; out hr: HRESULT); overload;
    constructor CreateFromFactory(Factory: TBCClassFactory; const Controller: IUnknown); override;
    destructor Destroy; override;
    function NonDelegatingQueryInterface(const IID: TGUID; out Obj): HResult; override; stdcall;
    // IFileSourceFilter
    function Load(pszFileName: PWCHAR; const pmt: PAMMediaType): HResult; stdcall;
    function GetCurFile(out ppszFileName: PWideChar; pmt: PAMMediaType): HResult; stdcall;
    // IAMMediaContent
    function get_AuthorName(var pbstrAuthorName: TBSTR): HResult; stdcall;
    function get_Title(var pbstrTitle: TBSTR): HResult; stdcall;
    function get_Rating(var pbstrRating: TBSTR): HResult; stdcall;
    function get_Description(var pbstrDescription: TBSTR): HResult; stdcall;
    function get_Copyright(var pbstrCopyright: TBSTR): HResult; stdcall;
    function get_BaseURL(var pbstrBaseURL: TBSTR): HResult; stdcall;
    function get_LogoURL(var pbstrLogoURL: TBSTR): HResult; stdcall;
    function get_LogoIconURL(var pbstrLogoURL: TBSTR): HResult; stdcall;
    function get_WatermarkURL(var pbstrWatermarkURL: TBSTR): HResult; stdcall;
    function get_MoreInfoURL(var pbstrMoreInfoURL: TBSTR): HResult; stdcall;
    function get_MoreInfoBannerImage(var pbstrMoreInfoBannerImage: TBSTR): HResult; stdcall;
    function get_MoreInfoBannerURL(var pbstrMoreInfoBannerURL: TBSTR): HResult; stdcall;
    function get_MoreInfoText(var pbstrMoreInfoText: TBSTR): HResult; stdcall;
    // IDispatch
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
  end;

  // Since I need to setup some Registry Values when Un/Registering,
  // I call the BaseClass functions from this File after i setup my Values.
  function DllGetClassObject(const CLSID, IID: TGUID; var Obj): HResult; stdcall;
  function DllCanUnloadNow: HResult; stdcall;
  function DllRegisterServer: HResult; stdcall;
  function DllUnregisterServer: HResult; stdcall;

var
  InstanceCount: Integer = 0;

implementation

constructor TMODSource.Create(const Name: string; unk: IUnknown; const clsid: TGUID; out hr: HRESULT);
begin
  inherited Create(Name,unk,CLSID_MODSource,hr);
  InterlockedIncrement(InstanceCount);
end;

constructor TMODSource.CreateFromFactory(Factory: TBCClassFactory; const Controller: IUnknown);
var
  hr: HRESULT;
begin
  Create(Factory.Name, Controller,CLSID_MODSource, hr);
end;

destructor TMODSource.Destroy;
begin
  InterlockedDecrement(InstanceCount);
  if Assigned(fFileName) then
  begin
    CoTaskMemFree(fFileName);
    fFileName := nil;
  end;
  if (fPin <> nil) then FreeAndNil(fPin);
  inherited;
end;

function TMODSource.NonDelegatingQueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
begin
  if IsEqualGuid(iid, IID_IFileSourceFilter) or IsEqualGuid(iid, IID_IAMFilterMiscFlags) then
  begin
    if GetInterface(IID, Obj) then Result := S_OK
                              else Result := E_NOINTERFACE;
  end
  else Result := inherited NonDelegatingQueryInterface(iid, Obj);
end;

function TMODSource.Load(pszFileName: PWCHAR; const pmt: PAMMediaType): HResult; stdcall;
var
  hr : HRESULT;
begin
	if GetPinCount > 0 then
  begin
		Result := VFW_E_ALREADY_CONNECTED;
    Exit;
  end;

  FPin := TMODSourceStream.Create('MOD Source Stream',hr,Self,'Output',pszFileName);
	if Failed(hr) or (fPin = nil) then
  begin
    Result := hr;
    Exit;
  end;

  AMGetWideString(pszFileName,fFilename);
  Result := S_OK;
end;

function TMODSource.GetCurFile(out ppszFileName: PWideChar; pmt: PAMMediaType): HResult; stdcall;
begin
  if not Assigned(@ppszFileName) then
  begin
    Result := E_POINTER;
    Exit;
  end;

  if not Assigned(fFileName) or (fFilename  = '') then
  begin
    ppszFileName := nil;
    Result := E_FAIL;
    Exit;
  end;
  Result := AMGetWideString(fFileName,ppszFileName);
end;

function TMODSource.get_AuthorName(var pbstrAuthorName: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_Title(var pbstrTitle: TBSTR): HResult; stdcall;
begin
  if not Assigned(@pbstrTitle) then
  begin
    Result := E_POINTER;
    Exit;
  end;

  if fPin.fDecoder.Title = '' then
  begin
    Result := VFW_E_NOT_FOUND;
    Exit;
  end;

  pbstrTitle := SysAllocString(PWideChar(WideString(fPin.fDecoder.Title)));
  Result := S_OK;
end;

function TMODSource.get_Rating(var pbstrRating: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_Description(var pbstrDescription: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_Copyright(var pbstrCopyright: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_BaseURL(var pbstrBaseURL: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_LogoURL(var pbstrLogoURL: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_LogoIconURL(var pbstrLogoURL: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_WatermarkURL(var pbstrWatermarkURL: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_MoreInfoURL(var pbstrMoreInfoURL: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_MoreInfoBannerImage(var pbstrMoreInfoBannerImage: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_MoreInfoBannerURL(var pbstrMoreInfoBannerURL: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.get_MoreInfoText(var pbstrMoreInfoText: TBSTR): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.GetTypeInfoCount(out Count: Integer): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TMODSource.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

{*** TMODSourceStream *******************************************************}

constructor TMODSourceStream.Create(const ObjectName: string; out hr: HRESULT; Filter: TBCSource; const Name: WideString; Filename : PWCHAR);
begin
  inherited Create(ObjectName,hr,Filter,Name);
  if Failed(hr) then Exit;
  fDecoder := TMODDecoder.Create(Filename,hr);
  if hr <> S_OK then
  begin
    hr := E_FAIL;
    Exit;
  end;
  FRateSeeking := 1.0;
  FLock := TBCCritSec.Create;

  FSeekingCaps := AM_SEEKING_CanSeekForwards or AM_SEEKING_CanSeekBackwards or
                  AM_SEEKING_CanSeekAbsolute or AM_SEEKING_CanGetStopPos or AM_SEEKING_CanGetDuration;

  FStop := fDecoder.Duration;
  FDuration := FStop;
  FStart := 0;

  FDiscontinuity := False;
  FSampleTime := 0;
  FMediaTime := 0;

  FBufferSize := 5000;
end;

destructor TMODSourceStream.Destroy;
begin
  FreeAndNil(fDecoder);
  if Assigned(FLock)
    then FreeAndNil(FLock);
  inherited Destroy;
end;

function TMODSourceStream.NonDelegatingQueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
begin
  if IsEqualGuid(iid, IID_IMediaSeeking) then
  begin
    if GetInterface(IID, Obj) then Result := S_OK
                              else Result := E_NOINTERFACE;
  end
    else Result := inherited NonDelegatingQueryInterface(iid, Obj);
end;

function TMODSourceStream.DecideBufferSize(Allocator: IMemAllocator; Properties: PAllocatorProperties): HRESULT;
var
  Actual: ALLOCATOR_PROPERTIES;
begin
  if (Allocator = nil) or (Properties = nil) then
  begin
    Result := E_POINTER;
    Exit;
  end;

  FFilter.StateLock.Lock;
  try
    Properties.cBuffers := 1;
    Properties.cbBuffer := FBufferSize;

    Result := Allocator.SetProperties(Properties^, Actual);
    if Failed(Result) then Exit;
    // Is this allocator unsuitable?
    if (Actual.cbBuffer < Properties.cbBuffer) then Result := E_FAIL
                                               else Result := S_OK;
  finally
    FFilter.StateLock.UnLock;
  end;
end;

function TMODSourceStream.GetTimeFromBytes(Bytes : Int64) : TReferenceTime;
begin
  Result := Bytes * One_Second div fDecoder.BytesPerSample div fDecoder.Frequency div fDecoder.Channels;
end;

function TMODSourceStream.OnThreadStartPlay: HRESULT;
begin
  FDiscontinuity := True;
  Result := DeliverNewSegment(FStart, FStop, FRateSeeking);
end;

function TMODSourceStream.ChangeStart: HRESULT;
begin
  FSampleTime := 0;
  FMediaTime := FStart;
  UpdateFromSeek;
  Result := S_OK;
end;

function TMODSourceStream.ChangeStop: HRESULT;
begin
  UpdateFromSeek;
  Result := S_OK;
end;

function TMODSourceStream.ChangeRate: HRESULT;
begin
  Result := S_OK;  // ???
  FLock.Lock;
  try
    if (FRateSeeking <= 0) then
    begin
      FRateSeeking := 1.0;
      Result := E_FAIL;
      Exit;
     end;
  finally
    FLock.UnLock;
  end;
  UpdateFromSeek;
  Result := S_OK;
end;

procedure TMODSourceStream.UpdateFromSeek;
begin
  if (FThread.ThreadExists) then
  begin
    DeliverBeginFlush;
    Stop;
    FDecoder.Position := FStart;
    DeliverEndFlush;
    Run;
  end
    else FDecoder.Position := FStart;
end;

function TMODSourceStream.FillBuffer(Samp: IMediaSample): HRESULT;
var
  Buffer: PByte;
  Received: integer;
  TimeStart, TimeStop: Int64;
  SampleTime: Int64;
begin
  FLock.Lock;
  try
    if (FMediaTime >= FStop) then
    begin
      Result := S_FALSE;
      Exit;
    end;

    Samp.GetPointer(Buffer);

    Received := FDecoder.GetBuffer(PChar(Buffer), FBufferSize);

    if (Received <= 0) then
    begin
      Result := S_FALSE;
      Exit;
    end;

    Samp.SetActualDataLength(Received);

    SampleTime := GetTimeFromBytes(Received);
    TimeStart := FSampleTime;
    FSampleTime := FSampleTime + SampleTime;
    TimeStop := FSampleTime;
    Samp.SetTime(@TimeStart, @TimeStop);

    TimeStart := FMediaTime;
    FMediaTime := FMediaTime + SampleTime;
    TimeStop := FMediaTime;
    Samp.SetMediaTime(@TimeStart, @TimeStop);

    Samp.SetSyncPoint(True);

    if (FDiscontinuity) then
    begin
      Samp.SetDiscontinuity(True);
      FDiscontinuity := False;
    end;

    Result := S_OK;
  finally
    FLock.UnLock;
  end;
end;

function TMODSourceStream.GetMediaType(MediaType: PAMMediaType): HRESULT;
begin
  if (MediaType = nil) then
  begin
    Result := E_FAIL;
    Exit;
  end;

  FFilter.StateLock.Lock;
  try
    MediaType.majortype := MEDIATYPE_Audio;
    MediaType.subtype := MEDIASUBTYPE_PCM;
    MediaType.formattype := FORMAT_WaveFormatEx;
    MediaType.cbFormat := SizeOf(TWaveFormatEx);
    MediaType.pbFormat := CoTaskMemAlloc(MediaType.cbFormat);
    MediaType.lSampleSize := fDecoder.Channels * fDecoder.BytesPerSample;
    MediaType.bFixedSizeSamples := True;
    MediaType.bTemporalCompression := False;
    with PWaveFormatEx(MediaType.pbFormat)^ do
    begin
      wFormatTag := WAVE_FORMAT_PCM;
      nChannels := fDecoder.Channels;
      nSamplesPerSec := fDecoder.Frequency;
      wBitsPerSample := fDecoder.BitsPerSample;
      nBlockAlign := fDecoder.Channels * fDecoder.BytesPerSample;
      nAvgBytesPerSec := nSamplesPerSec * nBlockAlign;
      cbSize := 0;
    end;
    Result := S_OK;
  finally
    FFilter.StateLock.UnLock;
  end;
end;

function TMODSourceStream.GetCapabilities(out pCapabilities: DWORD): HResult;
begin
  pCapabilities := FSeekingCaps;
  Result := S_OK;
end;

function TMODSourceStream.CheckCapabilities(var pCapabilities: DWORD): HResult;
begin
  if BOOL((not FSeekingCaps) and pCapabilities)
    then Result := S_FALSE
    else Result := S_OK;
end;

function TMODSourceStream.IsFormatSupported(const pFormat: TGUID): HResult;
begin
  if IsEqualGUID(TIME_FORMAT_MEDIA_TIME,pFormat)
    then Result := S_OK
    else Result := S_FALSE;
end;

function TMODSourceStream.QueryPreferredFormat(out pFormat: TGUID): HResult;
begin
  pFormat := TIME_FORMAT_MEDIA_TIME;
  Result := S_OK;
end;

function TMODSourceStream.GetTimeFormat(out pFormat: TGUID): HResult;
begin
  pFormat := TIME_FORMAT_MEDIA_TIME;
  Result := S_OK;
end;

function TMODSourceStream.IsUsingTimeFormat(const pFormat: TGUID): HResult;
begin
  if IsEqualGUID(TIME_FORMAT_MEDIA_TIME,pFormat)
    then Result := S_OK
    else Result := S_FALSE;
end;

function TMODSourceStream.SetTimeFormat(const pFormat: TGUID): HResult;
begin
  if IsEqualGUID(TIME_FORMAT_MEDIA_TIME,pFormat)
    then Result := S_OK
    else Result := E_INVALIDARG;
end;

function TMODSourceStream.GetDuration(out pDuration: int64): HResult;
begin
  FLock.Lock;
  try
    pDuration := FDuration;
    Result := S_OK;
  finally
    FLock.UnLock;
  end;
end;

function TMODSourceStream.GetStopPosition(out pStop: int64): HResult;
begin
  FLock.Lock;
  try
    pStop := FStop;
    Result := S_OK;
  finally
    FLock.UnLock;
  end;
end;

function TMODSourceStream.GetCurrentPosition(out pCurrent: int64): HResult;
begin
  Result := E_NOTIMPL;
end;

function TMODSourceStream.ConvertTimeFormat(out pTarget: int64; pTargetFormat: PGUID; Source: int64; pSourceFormat: PGUID): HResult;
begin
  if ((not Assigned(pTargetFormat) or IsEqualGUID(pTargetFormat^,TIME_FORMAT_MEDIA_TIME)) and
      (not Assigned(pSourceFormat) or IsEqualGUID(pSourceFormat^,TIME_FORMAT_MEDIA_TIME))) then
  begin
    pTarget := Source;
    Result := S_OK;
  end else
    Result := E_INVALIDARG;
end;

function TMODSourceStream.SetPositions(var pCurrent: int64; dwCurrentFlags: DWORD; var pStop: int64; dwStopFlags: DWORD): HResult;
var
  StopPosBits,
  StartPosBits: Cardinal;
begin
  StopPosBits := dwStopFlags and AM_SEEKING_PositioningBitsMask;
  StartPosBits := dwCurrentFlags and AM_SEEKING_PositioningBitsMask;

  if (dwStopFlags > 0) then
  begin
    if(StopPosBits <> dwStopFlags) then
    begin
      Result := E_INVALIDARG;
      Exit;
     end;
  end;

  if (dwCurrentFlags > 0) then
  begin
    if ((StartPosBits <> AM_SEEKING_AbsolutePositioning) and (StartPosBits <> AM_SEEKING_RelativePositioning)) then
    begin
      Result := E_INVALIDARG;
      Exit;
    end;
  end;

  FLock.Lock;
  try
    if (StartPosBits = AM_SEEKING_AbsolutePositioning) then
    begin
      FStart := pCurrent;
    end
    else if (StartPosBits = AM_SEEKING_RelativePositioning) then
    begin
      FStart := FStart + pCurrent;
    end;
    if (StopPosBits = AM_SEEKING_AbsolutePositioning) then
    begin
      FStop := pStop;
    end
    else if (StopPosBits = AM_SEEKING_IncrementalPositioning) then
    begin
      FStop := FStart + pStop;
    end
    else if (StopPosBits = AM_SEEKING_RelativePositioning) then
    begin
      FStop := FStop + pStop;
    end;
  finally
    FLock.UnLock;
  end;

  Result := S_OK;
  if ((DWORD(SUCCEEDED(Result)) and StopPosBits) > 0) then
  begin
    Result := ChangeStop;
  end;
  if(StartPosBits > 0) then
  begin
    Result := ChangeStart;
  end;
end;

function TMODSourceStream.GetPositions(out pCurrent, pStop: int64): HResult;
begin
  pCurrent := FStart;
  pStop := FStop;
  Result := S_OK;
end;

function TMODSourceStream.GetAvailable(out pEarliest, pLatest: int64): HResult;
begin
  pEarliest := 0;
  FLock.Lock;
  try
    pLatest := FDuration;
  finally
    FLock.UnLock;
  end;
  Result := S_OK;
end;

function TMODSourceStream.SetRate(dRate: double): HResult;
begin
  FLock.Lock;
  try
    FRateSeeking := dRate;
  finally
    FLock.UnLock;
  end;
  Result := ChangeRate;
end;

function TMODSourceStream.GetRate(out pdRate: double): HResult;
begin
  FLock.Lock;
  try
    pdRate := FRateSeeking;
    Result := S_OK;
  finally
    FLock.UnLock;
  end;
end;

function TMODSourceStream.GetPreroll(out pllPreroll: int64): HResult;
begin
  pllPreroll := 0;
  Result := S_OK;
end;

{*** Registering and FileAssociation *******************************************}

function DllGetClassObject(const CLSID, IID: TGUID; var Obj): HResult; stdcall;
begin
  Result := BaseClass.DllGetClassObject(CLSID,IID,Obj);
end;

function DllCanUnloadNow: HResult; stdcall;
begin
  Result := BaseClass.DllCanUnloadNow;
end;

function DllRegisterServer: HResult; stdcall;
begin
  with TRegistry.Create do
  begin
    RootKey := HKEY_CLASSES_ROOT;
    // Setup DirectShow to use our Filesource for our Extensions
    // MO3 / IT / XM / S3M / MTM / MOD / UMX
    if OpenKey('Media Type\Extensions\.mo3',True) then
    begin
      WriteString('Source Filter',GUIDToString(CLSID_MODSource));
      CloseKey;
    end;
    if OpenKey('Media Type\Extensions\.it',True) then
    begin
      WriteString('Source Filter',GUIDToString(CLSID_MODSource));
      CloseKey;
    end;
    if OpenKey('Media Type\Extensions\.xm',True) then
    begin
      WriteString('Source Filter',GUIDToString(CLSID_MODSource));
      CloseKey;
    end;
    if OpenKey('Media Type\Extensions\.s3m',True) then
    begin
      WriteString('Source Filter',GUIDToString(CLSID_MODSource));
      CloseKey;
    end;
    if OpenKey('Media Type\Extensions\.mtm',True) then
    begin
      WriteString('Source Filter',GUIDToString(CLSID_MODSource));
      CloseKey;
    end;
    if OpenKey('Media Type\Extensions\.mod',True) then
    begin
      WriteString('Source Filter',GUIDToString(CLSID_MODSource));
      CloseKey;
    end;
    if OpenKey('Media Type\Extensions\.umx',True) then
    begin
      WriteString('Source Filter',GUIDToString(CLSID_MODSource));
      CloseKey;
    end;
    Free;
  end;
  Result := BaseClass.DllRegisterServer;
end;

function DllUnregisterServer: HResult; stdcall;
begin
  with TRegistry.Create do
  begin
    RootKey := HKEY_CLASSES_ROOT;
    // MO3 / IT / XM / S3M / MTM / MOD / UMX
    if KeyExists('Media Type\Extensions\.mo3') then DeleteKey('Media Type\Extensions\.mo3');
    if KeyExists('Media Type\Extensions\.it') then DeleteKey('Media Type\Extensions\.it');
    if KeyExists('Media Type\Extensions\.xm') then DeleteKey('Media Type\Extensions\.xm');
    if KeyExists('Media Type\Extensions\.s3m') then DeleteKey('Media Type\Extensions\.s3m');
    if KeyExists('Media Type\Extensions\.mtm') then DeleteKey('Media Type\Extensions\.mtm');
    if KeyExists('Media Type\Extensions\.mod') then DeleteKey('Media Type\Extensions\.mod');
    if KeyExists('Media Type\Extensions\.umx') then DeleteKey('Media Type\Extensions\.umx');
    Free;
  end;
  Result := BaseClass.DllUnregisterServer;
end;

initialization

  TBCClassFactory.CreateFilter(TMODSource, 'File Source (MO3/XM/IT)', CLSID_MODSource,
    CLSID_LegacyAmFilterCategory, MERIT_UNLIKELY, 1, @MyPins);

end.
